from setuptools import setup


setup(
    name="pymask-gyl",
    version="0.0.3",
    author="charles",
    author_email="endlesslights@163.com",
    url="https://github.com",
    description=u"tools for myself",
    packages=['pymask'],
    install_requires=["numpy"],
)